#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = 'flowsolution unstructured'
diag = True


def makeUnstTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, 'Base', 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype='int32', order='F')
    z = CGL.newZone(b, 'Zone', s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, 'GridCoordinates')
    d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s, NPY.array([[1, cellsize]], 'i', order='F'),
                             NPY.ones((cellsize * 4), dtype='int32'))
    return (T, b, z)


vertexsize = 20
cellsize = 7
(T, b, z) = makeUnstTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name='sol1', gridlocation=CGK.Vertex_s)
CGL.newDataArray(sol1, 'var', value=NPY.ones(vertexsize, dtype='float64', order='F'))
sol2 = CGL.newFlowSolution(z, name='sol2', gridlocation=CGK.CellCenter_s)
CGL.newDataArray(sol2, 'var', value=NPY.ones(cellsize, dtype='float64', order='F'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'flowsolution bad dataarray dim'
diag = False
(T, b, z) = makeUnstTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name='sol1', gridlocation=CGK.Vertex_s)
CGL.newDataArray(sol1, 'var', value=NPY.ones(vertexsize + 1, dtype='float64', order='F'))
sol2 = CGL.newFlowSolution(z, name='sol2', gridlocation=CGK.CellCenter_s)
CGL.newDataArray(sol2, 'var', value=NPY.ones(cellsize + 1, dtype='float64', order='F'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'flowsolution structured'
diag = True


def makeStTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, '{Base}', 3, 3)
    z = CGL.newZone(b, '{Zone}', NPY.array(
        [[vertexsize[0], cellsize[0], 0], [vertexsize[1], cellsize[1], 0], [vertexsize[2], cellsize[2], 0]], order='F'))
    g = CGL.newGridCoordinates(z, 'GridCoordinates')
    d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones(tuple(vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones(tuple(vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones(tuple(vertexsize), dtype='float64', order='F'))
    return (T, b, z)


vertexsize = [5, 7, 9]
cellsize = [i - 1 for i in vertexsize]
(T, b, z) = makeStTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name='sol1', gridlocation=CGK.Vertex_s)
CGL.newDataArray(sol1, 'var', value=NPY.ones(tuple(vertexsize), dtype='float64', order='F'))
sol2 = CGL.newFlowSolution(z, name='sol2', gridlocation=CGK.CellCenter_s)
CGL.newDataArray(sol2, 'var', value=NPY.ones(tuple(cellsize), dtype='float64', order='F'))
sol3 = CGL.newFlowSolution(z, name='sol3', gridlocation=CGK.Vertex_s)
CGL.newRind(sol3, NPY.ones(2 * 3, dtype='int32', order='F'))
dsize = [i + 1 + 1 for i in vertexsize]
CGL.newDataArray(sol3, 'var', value=NPY.ones(tuple(dsize), dtype='float64', order='F'))
sol4 = CGL.newFlowSolution(z, name='sol4', gridlocation=CGK.CellCenter_s)
CGL.newRind(sol4, NPY.ones(2 * 3, dtype='int32', order='F'))
dsize = [i + 1 + 1 for i in cellsize]
CGL.newDataArray(sol4, 'var', value=NPY.ones(tuple(dsize), dtype='float64', order='F'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'flowsolution bad dataarray dim'
diag = False
(T, b, z) = makeStTree(vertexsize, cellsize)
sol1 = CGL.newFlowSolution(z, name='sol1', gridlocation=CGK.Vertex_s)
dsize = [i + 1 for i in vertexsize]
CGL.newDataArray(sol1, 'var', value=NPY.ones(tuple(dsize), dtype='float64', order='F'))
sol2 = CGL.newFlowSolution(z, name='sol2', gridlocation=CGK.CellCenter_s)
dsize = [i + 1 for i in cellsize]
CGL.newDataArray(sol2, 'var', value=NPY.ones(tuple(dsize), dtype='float64', order='F'))
sol3 = CGL.newFlowSolution(z, name='sol3', gridlocation=CGK.Vertex_s)
CGL.newRind(sol3, NPY.ones(2 * 3, dtype='int32', order='F'))
dsize = [i for i in vertexsize]
CGL.newDataArray(sol3, 'var', value=NPY.ones(tuple(dsize), dtype='float64', order='F'))
sol4 = CGL.newFlowSolution(z, name='sol4', gridlocation=CGK.CellCenter_s)
CGL.newRind(sol4, NPY.ones(2 * 3, dtype='int32', order='F'))
dsize = [i for i in cellsize]
CGL.newDataArray(sol4, 'var', value=NPY.ones(tuple(dsize), dtype='float64', order='F'))
TESTS.append((tag, T, diag))



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  ---------------------------------------------------------------------------
#
from __future__ import unicode_literals
from __future__ import absolute_import
from . import CGNSBase_t
from . import Zone_t
from . import ZoneBC_t
from . import BC_t
from . import DataClass_t
from . import Descriptor_t
from . import DimensionalUnits_t
from . import DimensionalExponents_t
from . import GridLocation_t
from . import PointList_t
from . import PointRange_t
from . import Rind_t
from . import DataConversion_t
from . import SimulationType_t
from . import Ordinal_t
from . import GridCoordinates_t
from . import DataArray_t
from . import DiscreteData_t
from . import Elements_t
from . import BCDataSet_t
from . import BCData_t
from . import BCProperty_t
from . import RotatingCoordinates_t
from . import ZoneGridConnectivity_t
from . import GridConnectivity1to1_t
from . import GridConnectivity_t
from . import GridConnectivityProperty_t
from . import AverageInterface_t
from . import OversetHoles_t
from . import FlowSolution_t
from . import FlowEquationSet_t
from . import GoverningEquations_t
from . import GasModel_t
from . import ThermalConductivityModel_t
from . import ThermalRelaxationModel_t
from . import ChemicalKineticModel_t
from . import EMElectricFieldModel_t
from . import EMMagneticFieldModel_t
from . import EMConductivityModel_t
from . import ViscosityModel_t
from . import TurbulenceClosure_t
from . import TurbulenceModel_t
from . import AxiSymmetry_t
from . import BaseIterativeData_t
from . import ZoneIterativeData_t
from . import RigidGridMotion_t
from . import ArbitraryGridMotion_t
from . import ReferenceState_t
from . import ConvergenceHistory_t
from . import IntegralData_t
from . import UserDefinedData_t
from . import Gravity_t
from . import Family_t
from . import FamilyName_t
from . import FamilyBC_t
from . import GeometryReference_t
from . import ZoneSubRegion_t

profile = {
    'CGNSBase_t': CGNSBase_t.pattern,
    'Zone_t': Zone_t.pattern,
    'ZoneBC_t': ZoneBC_t.pattern,
    'BC_t': BC_t.pattern,
    'DataClass_t': DataClass_t.pattern,
    'Descriptor_t': Descriptor_t.pattern,
    'DimensionalUnits_t': DimensionalUnits_t.pattern,
    'DimensionalExponents_t': DimensionalExponents_t.pattern,
    'GridLocation_t': GridLocation_t.pattern,
    'PointList_t': PointList_t.pattern,
    'PointRange_t': PointRange_t.pattern,
    'Rind_t': Rind_t.pattern,
    'DataConversion_t': DataConversion_t.pattern,
    'SimulationType_t': SimulationType_t.pattern,
    'Ordinal_t': Ordinal_t.pattern,
    'GridCoordinates_t': GridCoordinates_t.pattern,
    'DataArray_t': DataArray_t.pattern,
    'DiscreteData_t': DiscreteData_t.pattern,
    'Elements_t': Elements_t.pattern,
    'BCDataSet_t': BCDataSet_t.pattern,
    'BCData_t': BCData_t.pattern,
    'BCProperty_t': BCProperty_t.pattern,
    'RotatingCoordinates_t': RotatingCoordinates_t.pattern,
    'ZoneGridConnectivity_t': ZoneGridConnectivity_t.pattern,
    'GridConnectivity1to1_t': GridConnectivity1to1_t.pattern,
    'GridConnectivity_t': GridConnectivity_t.pattern,
    'GridConnectivityProperty_t': GridConnectivityProperty_t.pattern,
    'AverageInterface_t': AverageInterface_t.pattern,
    'OversetHoles_t': OversetHoles_t.pattern,
    'FlowSolution_t': FlowSolution_t.pattern,
    'FlowEquationSet_t': FlowEquationSet_t.pattern,
    'GoverningEquations_t': GoverningEquations_t.pattern,
    'GasModel_t': GasModel_t.pattern,
    'ThermalConductivityModel_t': ThermalConductivityModel_t.pattern,
    'ThermalRelaxationModel_t': ThermalRelaxationModel_t.pattern,
    'ChemicalKineticModel_t': ChemicalKineticModel_t.pattern,
    'EMElectricFieldModel_t': EMElectricFieldModel_t.pattern,
    'EMMagneticFieldModel_t': EMMagneticFieldModel_t.pattern,
    'EMConductivityModel_t': EMConductivityModel_t.pattern,
    'ViscosityModel_t': ViscosityModel_t.pattern,
    'TurbulenceClosure_t': TurbulenceClosure_t.pattern,
    'TurbulenceModel_t': TurbulenceModel_t.pattern,
    'AxiSymmetry_t': AxiSymmetry_t.pattern,
    'BaseIterativeData_t': BaseIterativeData_t.pattern,
    'ZoneIterativeData_t': ZoneIterativeData_t.pattern,
    'RigidGridMotion_t': RigidGridMotion_t.pattern,
    'ArbitraryGridMotion_t': ArbitraryGridMotion_t.pattern,
    'ReferenceState_t': ReferenceState_t.pattern,
    'ConvergenceHistory_t': ConvergenceHistory_t.pattern,
    'IntegralData_t': IntegralData_t.pattern,
    'UserDefinedData_t': UserDefinedData_t.pattern,
    'Gravity_t': Gravity_t.pattern,
    'Family_t': Family_t.pattern,
    'FamilyName_t': FamilyName_t.pattern,
    'FamilyBC_t': FamilyBC_t.pattern,
    'GeometryReference_t': GeometryReference_t.pattern,
    'ZoneSubRegion_t': ZoneSubRegion_t.pattern,
}
#



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
#
from __future__ import unicode_literals
import CGNS.VAL.simplecheck as CGV
import CGNS.MAP
import importlib
import sys
import os
import string

KEY = 'SAMPLE'
FILES = False
CGNSCHECK = False
TRACE = False

try:
    import CGNS.PRO

    KEY = 'SIDS'
except ImportError:
    pass

try:
    if 'CGNS_VAL_SAVE_FILES' in os.environ:
        FILES = True
except:
    pass

try:
    if 'CGNS_VAL_RUN_CGNSCHECK' in os.environ:
        CGNSCHECK = True
except:
    pass


def cgcheck(filenametag):
    os.system('cgnscheck %s.hdf 1>%s.cgnscheck 2>&1' % (filenametag, filenametag))


def runSuite(suite, trace, savefile=False, cgnscheck=False):
    s = importlib.import_module('.%s' % suite, 'CGNS.VAL.suite')
    tlist = []
    count = 1
    if s is not None:
        p = os.path.split(s.__file__)[0]
        l = os.listdir(p)
        for t in l:
            if ((t[0] != '_') and (t[-3:] == '.py')
                and (t[0] in string.digits)
                and (t[1] in string.digits)):
                tlist.append(t[:-3])
    for t in tlist:
        tdlist = loadTree(suite, t)
        for (tag, T, diag) in tdlist:
            sr = valTree(suite, t, tag, T, diag, trace, count)
            if diag:
                k = 'pass'
            else:
                k = 'fail'
            if (savefile):
                fname = '%.4d-%s-%s-%s.hdf' % (count, k, suite, t)
                try:
                    CGNS.MAP.save(fname, T)
                except CGNS.MAP.error as e:
                    sr += '### * CHLone save error %d:%s\n' % (e[0], e[1])
                f = open('%.4d-%s-%s-%s.diag' % (count, k, suite, t), 'w+')
                f.write(sr)
                f.close()
                if (cgnscheck):
                    cgcheck('%.4d-%s-%s-%s' % (count, k, suite, t))
            count += 1


def loadTree(key, test):
    m = importlib.import_module('.%s' % test, 'CGNS.VAL.suite.%s' % key)
    return (m.TESTS)


def valTree(suite, t, tag, T, diag, trace, count):
    sr = ''
    r = CGV.compliant(T, False, [KEY])
    if diag:
        k = 'pass'
    else:
        k = 'fail'
    if r[0] == diag:
        (v1, v2) = ('pass', 'expected')
    else:
        (v1, v2) = ('FAIL', 'UNEXPECTED')
    if trace:
        sr += '###\n'
    sr += '### [%s/%s] %.4d-%s-%s-%s (%s)\n' % (KEY, v1, count, k, suite, t, tag)
    for m in r[1]:
        sr += '### > %s error on: %s\n' % (v2, m[0])
        sr += '### > %s\n' % (m[1])
    if trace:
        print(sr)
    return sr


def runall():
    tlist = ('SIDS',)
    for s in tlist:
        KEY = s
        runSuite(s, TRACE, FILES, CGNSCHECK)



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

from CGNS.NAV.moption import Q7OptionContext as OCTXT

import sys
import string
import os.path

import CGNS.MAP as CGM
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK

from qtpy.QtCore import Qt
from qtpy.QtWidgets import QFileDialog
from qtpy.QtWidgets import (QTableWidgetItem, QStyledItemDelegate, QLineEdit, QHeaderView)

from CGNS.NAV.Q7LinkWindow import Ui_Q7LinkWindow
from CGNS.NAV.wfingerprint import Q7Window as QW

import CGNS.NAV.wmessages as MSG


# -----------------------------------------------------------------
class Q7LinkItemDelegate(QStyledItemDelegate):
    def __init__(self, owner, model):
        self._table = owner.linkTable
        QStyledItemDelegate.__init__(self, self._table)
        self._parent = owner
        self._model = model
        self._filename = ''
        self._dirname = ''

    def createEditor(self, parent, option, index):
        ws = option.rect.width()
        hs = option.rect.height() + 4
        xs = option.rect.x()
        ys = option.rect.y() - 2
        if index.column() in [2]:
            filt = "CGNS Files (*.hdf *.cgns)"
            filename = str(QFileDialog.getOpenFileName(self._table,
                                                       "Select file",
                                                       filter=filt)[0])
            (dname, fname) = os.path.split(filename)
            if not dname or not fname:
                return None
            itf = QTableWidgetItem(fname)
            itd = QTableWidgetItem(dname)
            self._table.setItem(index.row(), 2, itf)
            self._table.setItem(index.row(), 4, itd)
            return None
        if index.column() in [1, 3]:
            editor = QLineEdit(parent)
            editor.transgeometry = (xs, ys, ws, hs)
            editor.installEventFilter(self)
            self.setEditorData(editor, index)
            return editor
        return None

    def setEditorData(self, editor, index):
        value = index.data()
        editor.clear()
        editor.insert(value)

    def setModelData(self, editor, model, index):
        col = index.column()
        row = index.row()
        v = str(editor.text())
        if col in [1, 3]:
            v = CGU.getPathNoRoot(v)
        self._parent._links[row][self._parent._col2lk[col]] = v
        self._parent.reset()

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(*editor.transgeometry)

    def paint(self, painter, option, index):
        QStyledItemDelegate.paint(self, painter, option, index)


# -----------------------------------------------------------------
class Q7LinkList(QW, Ui_Q7LinkWindow):
    def __init__(self, parent, fgindex, master):
        QW.__init__(self, QW.VIEW_LINK, parent, None, fgindex)
        self.bClose.clicked.connect(self.reject)
        self._lk2col = [4, 2, 3, 1, 0]
        self._col2lk = [4, 3, 1, 2, 0]
        self._master = master
        self._links = self.FG.links
        self.linkTable.setItemDelegate(Q7LinkItemDelegate(self,
                                                          self.FG.model))
        self.setLabel(self.eDirSource, self.FG.filedir)
        self.setLabel(self.eFileSource, self.FG.filename)
        self.bInfo.clicked.connect(self.infoLinkView)
        self.bAddLink.clicked.connect(self.newLink)
        self.bDeleteLink.clicked.connect(self.removeLink)
        self.bDuplicateLink.clicked.connect(self.duplicateLink)
        self.bSave.clicked.connect(self.infoLinkView)
        self.bCheckLink.clicked.connect(self.checkLinks)
        self.bLoadTree.clicked.connect(self.loadLinkFile)

    def doRelease(self):
        self.reject()

    def loadLinkFile(self):
        i = self.linkTable.currentItem()
        if i is None:
            return
        r = i.row()
        d = self.linkTable.item(r, 4).text()
        f = self.linkTable.item(r, 2).text()
        filename = "%s/%s" % (d, f)
        self.busyCursor()
        if filename is not None:
            self._control.loadfile(filename)
        self.readyCursor()

    def infoLinkView(self):
        self._control.helpWindow('Link')

    def duplicateLink(self):
        if self._links:
            i = self.linkTable.currentItem()
            if i is None:
                return
            r = i.row()
            self._links.append(self._links[r])
            self.reset()
        return

    def removeLink(self):
        if not self._links:
            return
        i = self.linkTable.currentItem()
        if i is None:
            return
        reply = MSG.wQuestion(self, 311, 'Remove link entry',
                              """Do you want to remove the selected link entry?
                                 Existing sub-tree would be <b>merged</b> in the
                                 top file during the next save.""")
        if reply:
            r = i.row()
            self._links.pop(r)
            self.reset()

    def checkLinks(self):
        pass

    def newLink(self):
        self._links.append(['', '', '', '', CGM.S2P_LKIGNORED])
        self.reset()

    def show(self):
        self.reset()
        super(Q7LinkList, self).show()

    def statusIcon(self, status):
        if status == CGM.S2P_LKOK:
            it = QTableWidgetItem(self.IC(QW.I_L_OKL), '')
            it.setToolTip('Link ok')
            return it
        if status & CGM.S2P_LKNOFILE:
            it = QTableWidgetItem(self.IC(QW.I_L_NFL), '')
            it.setToolTip('File not found in search path')
            return it
        if status & CGM.S2P_LKIGNORED:
            it = QTableWidgetItem(self.IC(QW.I_L_IGN), '')
            it.setToolTip('Link was ignored during load')
            return it
        if status & CGM.S2P_LKFILENOREAD:
            it = QTableWidgetItem(self.IC(QW.I_L_NRL), '')
            it.setToolTip('File found, not readable')
            return it
        if status & CGM.S2P_LKNONODE:
            it = QTableWidgetItem(self.IC(QW.I_L_NNL), '')
            it.setToolTip('File ok, node path not found')
            return it
        it = QTableWidgetItem(self.IC(QW.I_L_ERL), '')
        it.setToolTip('Unknown error')
        return it

    def reset(self):
        v = self.linkTable
        v.clear()
        v.setRowCount(0)
        lh = v.horizontalHeader()
        lv = v.verticalHeader()
        h = ['S', 'Source Node', 'Linked-to file', 'Linked-to Node', 'Found in dir']
        for i, hstr in enumerate(h):
            hi = QTableWidgetItem(hstr)
            hi.setFont(OCTXT._Label_Font)
            v.setHorizontalHeaderItem(i, hi)
            lh.setSectionResizeMode(i, QHeaderView.ResizeToContents)
        lh.setSectionResizeMode(len(h) - 1, QHeaderView.Stretch)
        v.setRowCount(len(self._links))
        r = 0
        for lk in self._links:
            (ld, lf, ln, sn, st) = lk
            t1item = self.statusIcon(st)
            t2item = QTableWidgetItem(sn)
            t2item.setFont(OCTXT._Table_Font)
            t3item = QTableWidgetItem(lf)
            t3item.setFont(OCTXT._Table_Font)
            t4item = QTableWidgetItem(ln)
            t4item.setFont(OCTXT._Table_Font)
            t5item = QTableWidgetItem(ld)
            t5item.setFont(OCTXT._Table_Font)
            v.setItem(r, 0, t1item)
            v.setItem(r, 1, t2item)
            v.setItem(r, 2, t3item)
            v.setItem(r, 3, t4item)
            v.setItem(r, 4, t5item)
            r += 1
        for i in (2, 3):
            v.resizeColumnToContents(i)
        for i in range(v.rowCount()):
            v.resizeRowToContents(i)

    def reject(self):
        if self._master._linkwindow is not None:
            self._master._linkwindow = None
        self.close()

# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS.NAV - Python package for CFD General Notation System - NAVigater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

#from CGNS.NAV.moption import Q7OptionContext as OCTXT

import re

from qtpy.QtWidgets import QTextEdit
from qtpy.QtGui import (QColor, QSyntaxHighlighter, QTextCursor, QFont, QTextCharFormat)


# -----------------------------------------------------------------
class Q7DocEditor(QTextEdit):
    def __init__(self, parent=None):
        QTextEdit.__init__(self, parent)

    def initText(self, text):
        self.clear()
        self.insertPlainText(text)
        self.moveCursor(QTextCursor.Start, QTextCursor.MoveAnchor)
        self.ensureCursorVisible()


# -----------------------------------------------------------------
class Q7PythonEditor(QTextEdit):
    def __init__(self, parent=None):
        QTextEdit.__init__(self, parent)
        self.highlighter = Q7PythonEditorHighlighter(self.document())
        fsz = 12
        ffm = 'Courier'
        self.setStyleSheet("font: %dpt \"%s\";" % (fsz, ffm))

    def initText(self, text):
        self.clear()
        self.insertPlainText(text)
        self.moveCursor(QTextCursor.Start, QTextCursor.MoveAnchor)
        self.ensureCursorVisible()


# -----------------------------------------------------------------
class Q7PythonEditorHighlighter(QSyntaxHighlighter):
    def __init__(self, *args):
        keywords = r"\bimport\b|\bas\b|\bfor\b|\bdef\b|\bpass\b"
        keywords += r"|\bclass\b|\bfrom\b|\bif\b|\bthen\b|\belse\b"
        keywords += r"|\belif\b|\btry\b|\bexcept\b|\bfinally\b|\braise\b"
        keywords += r"|\bprint\b|\bin\b|\bnot\b|\band\b|\bor\b|\bcontinue\b"
        keywords += r"|\bwhile\b|\breturn\b|\blambda\b|\bwith\b|\del\b"
        keywords += r"|\bglobal\b|\byield\b|\bexec\b|\bassert\b|\break\b"
        constants = r"\bNone\b|\bTrue\b|\bFalse\b|\bself\b"
        autovars = r"\bNODE\b|\bNAME\b|\bVALUE\b|\bSIDSTYPE\b|\bCHILDREN\b"
        autovars += r"|\bTREE\b|\bPATH\b|\bRESULT\b|\bARGS\b|\bPARENT\b"
        autovars += r"|\bLINKS\b|\bSKIPS\b|\bSELECTED\b"
        numbers = r'[-+]?\d+' + '|[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?'
        comment = r'\#.*$'
        textstring = r'"[^"]*?"|\'[^\']*?\''
        self.f_keywords = QTextCharFormat()
        self.f_keywords.setFontWeight(QFont.Bold)
        self.f_keywords.setForeground(QColor('Blue'))
        self.f_comment = QTextCharFormat()
        self.f_comment.setFontWeight(QFont.Light)
        self.f_comment.setForeground(QColor('Red'))
        self.f_constants = QTextCharFormat()
        self.f_constants.setFontWeight(QFont.Light)
        self.f_constants.setForeground(QColor('Navy'))
        self.f_autovars = QTextCharFormat()
        self.f_autovars.setFontWeight(QFont.Bold)
        self.f_autovars.setForeground(QColor('Green'))
        self.f_textstring = QTextCharFormat()
        self.f_textstring.setFontWeight(QFont.Bold)
        self.f_textstring.setForeground(QColor('Coral'))
        self.f_numbers = QTextCharFormat()
        self.f_numbers.setFontWeight(QFont.Light)
        self.f_numbers.setForeground(QColor('Gray'))
        self.r_keywords = re.compile(keywords)
        self.r_numbers = re.compile(numbers)
        self.r_comment = re.compile(comment)
        self.r_constants = re.compile(constants)
        self.r_autovars = re.compile(autovars)
        self.r_textstring = re.compile(textstring)
        self.in_comment = False
        self.in_string = False
        self.r_syntax = []
        self.r_syntax.append((self.r_keywords, self.f_keywords))
        self.r_syntax.append((self.r_numbers, self.f_numbers))
        self.r_syntax.append((self.r_autovars, self.f_autovars))
        self.r_syntax.append((self.r_constants, self.f_constants))
        self.r_syntax.append((self.r_comment, self.f_comment))
        self.r_syntax.append((self.r_textstring, self.f_textstring))
        QSyntaxHighlighter.__init__(self, *args)

    def highlightBlock(self, textblock):
        text = str(textblock)
        fdef = self.format(0)
        fmax = len(text)
        for (rex, fmt) in self.r_syntax:
            mit = rex.finditer(text)
            for m in mit:
                self.setFormat(m.span()[0], m.span()[1], fmt)
                self.setFormat(m.span()[1], fmax, fdef)

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
from __future__ import unicode_literals
from __future__ import print_function


def run():
    import CGNS.VAL.test.cgu


def run2():
    print('### pyCGNS TEST: starting VAL tests')
    import CGNS.VAL.suite.run
    CGNS.VAL.suite.run.runall()

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
# actual versions number are in setuputils.py
#
from CGNS import pyCGNSconfig
__version__ = pyCGNSconfig.__version__
__release__ = pyCGNSconfig.__release__
__vid__ = pyCGNSconfig.__vid__
__doc__ = pyCGNSconfig.__doc__
version = __vid__
id = __vid__
#
# --- last line



#!/usr/bin/env python
from __future__ import print_function
import CGNS.MAP
import CGNS.PAT.cgnsutils as PU
import numpy
import time
import sys
numpy.set_printoptions(threshold=sys.maxint)

flags=CGNS.MAP.S2P_FOLLOWLINKS|CGNS.MAP.S2P_TRACE
start=time.clock()
(tree,links)=CGNS.MAP.load("./001Disk.hdf",flags)

node=PU.getNodeByPath(tree,"/Disk/zone1/ZoneBC/ext1/PointRange")

print("PointRange is fortran:",numpy.isfortran(node[1]),node[1],node[1].shape)

f=open('T0.py','w+')
f.write('from numpy import *\n')
f.write('tree=')
f.write(str(tree))
f.write('\nlinks=')
f.write(str(links))
f.write('\n')
f.close()

CGNS.MAP.save("./002Disk.hdf",tree,links,flags)

end=time.clock()
print('# time =',end-start)



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  ---------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib      as C
import CGNS.PAT.cgnserrors   as E
import CGNS.PAT.cgnskeywords as K
import numpy             as N

#RegionCellDimension
data = ['{ZoneSubRegion}', N.array([3],dtype=N.int32, order='F'),[], K.ZoneSubRegion_ts]

C.newGridLocation(data, K.Vertex_s)
C.newPointRange(data)
C.newFamilyName(data, '{Family}')
C.newUserDefinedData(data, '{UserDefinedData}')
C.newDescriptor(data, '{Descriptor}')

status = '7.3'
comment = 'partial SIDS with some optionals'
pattern = [data, status, comment]



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = 'indexrange'
diag = True


def makeCorrectTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, 'Base', 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype='int32', order='F')
    z = CGL.newZone(b, 'Zone', s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, 'GridCoordinates')
    d = CGL.newDataArray(g, CGK.CoordinateX_s,
                         NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateY_s,
                         NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateZ_s,
                         NPY.ones((vertexsize), dtype='float64', order='F'))
    return (T, b, z)


vertexsize = 20
cellsize = 7
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                         NPY.array([[1, cellsize]], 'i', order='F'),
                         NPY.ones((cellsize * 4), dtype='int32'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'indexrange index bad ordered'  # this raises a warning, not an error
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                         NPY.array([[cellsize, 1]], 'i', order='F'),
                         NPY.ones((cellsize * 4), dtype='int32'))
# element range not ordered
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'indexrange bad node shape'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                         NPY.array([1, cellsize], 'i', order='F'),
                         NPY.ones((cellsize * 4), dtype='int32'))
# ElementRange bad node shape
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'indexrange on BC_t'
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                         NPY.array([[1, cellsize]], 'i', order='F'),
                         NPY.ones((cellsize * 4), dtype='int32'))
ntris = 11
tris = CGL.newElements(z, 'TRIS', CGK.TRI_3_s,
                       NPY.array([[cellsize + 1, cellsize + ntris]], 'i', order='F'),
                       NPY.ones((ntris * 3), dtype='int32'))
n = CGL.newBoundary(z, 'BC', [range(cellsize + 1, cellsize + ntris + 1)], btype=CGK.Null_s, family=None,
                    pttype=CGK.PointList_s)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(CGK.PointRange_s, NPY.array([[cellsize + 1, cellsize + ntris]], dtype=NPY.int32, order='F'), [],
            CGK.IndexRange_ts, parent=n)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'indexrange on BC_t PointRange index out of range #1'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                         NPY.array([[1, cellsize]], 'i', order='F'),
                         NPY.ones((cellsize * 4), dtype='int32'))
ntris = 11
tris = CGL.newElements(z, 'TRIS', CGK.TRI_3_s,
                       NPY.array([[cellsize + 1, cellsize + ntris]], 'i', order='F'),
                       NPY.ones((ntris * 3), dtype='int32'))
n = CGL.newBoundary(z, 'BC', [range(cellsize + 1, cellsize + ntris + 1)],
                    btype=CGK.Null_s, family=None, pttype=CGK.PointList_s)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(CGK.PointRange_s,
            NPY.array([[cellsize + 1, cellsize + ntris + 1]], dtype=NPY.int32, order='F'), [], CGK.IndexRange_ts,
            parent=n)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'indexrange on BC_t PointRange index out of range #2'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                         NPY.array([[1, cellsize]], 'i', order='F'),
                         NPY.ones((cellsize * 4), dtype='int32'))
ntris = 11
tris = CGL.newElements(z, 'TRIS', CGK.TRI_3_s,
                       NPY.array([[cellsize + 1, cellsize + ntris]], 'i', order='F'),
                       NPY.ones((ntris * 3), dtype='int32'))
n = CGL.newBoundary(z, 'BC', [range(cellsize + 1, cellsize + ntris + 1)], btype=CGK.Null_s, family=None,
                    pttype=CGK.PointList_s)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(CGK.PointRange_s, NPY.array([[cellsize, cellsize + ntris]], dtype=NPY.int32, order='F'), [],
            CGK.IndexRange_ts, parent=n)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'indexrange on BC_t ElementRange index out of range'
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                         NPY.array([[1, cellsize]], 'i', order='F'),
                         NPY.ones((cellsize * 4), dtype='int32'))
ntris = 11
tris = CGL.newElements(z, 'TRIS', CGK.TRI_3_s,
                       NPY.array([[cellsize + 1, cellsize + ntris]], 'i', order='F'),
                       NPY.ones((ntris * 3), dtype='int32'))
n = CGL.newBoundary(z, 'BC', [range(cellsize, cellsize + ntris + 1)], btype=CGK.Null_s, family=None,
                    pttype=CGK.PointList_s)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
CGU.removeChildByName(n, CGK.PointList_s)
CGU.newNode(CGK.ElementRange_s, NPY.array([[cellsize + 1, cellsize + ntris + 1]], dtype=NPY.int32, order='F'), [],
            CGK.IndexRange_ts, parent=n)
TESTS.append((tag, T, diag))
